package cz.cvut.kbss.reporting.service;

import cz.cvut.kbss.reporting.environment.Environment;
import cz.cvut.kbss.reporting.model.Severity;
import cz.cvut.kbss.reporting.rest.util.RawJson;
import cz.cvut.kbss.reporting.util.Constants;
import org.hamcrest.Matcher;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.test.web.client.MockRestServiceServer;
import org.springframework.web.client.RestTemplate;

import java.net.URLEncoder;

import static org.hamcrest.CoreMatchers.containsString;
import static org.junit.Assert.*;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.method;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.requestTo;
import static org.springframework.test.web.client.response.MockRestResponseCreators.withSuccess;

public class OptionsServiceTest extends BaseServiceTestRunner {

    private static final String DATA = "[{\"a\": 1}, {\"a\": 2}]";

    @Rule
    public ExpectedException thrown = ExpectedException.none();

    @Autowired
    private OptionsService service;

    @Autowired
    private RestTemplate restTemplate;

    private MockRestServiceServer mockServer;

    @Before
    public void setUp() {
        this.mockServer = MockRestServiceServer.createServer(restTemplate);
    }

    @Test
    public void getOptionsUsesInternalLoaderToRetrieveSeverityOptions() {
        final Object result = service.getOptions(Severity.getOptionType());
        assertNotNull(result);
        assertTrue(result instanceof RawJson);
        final RawJson resJson = (RawJson) result;
        for (Severity s : Severity.values()) {
            assertThat(resJson.getValue(), containsString(s.toString()));
        }
    }

    @Test
    public void getOptionsUsesLocalLoaderToRetrieveOptionsFromOptionsFile() {
        final Object result = service.getOptions("testOptions");
        assertNotNull(result);
        assertTrue(result instanceof RawJson);
        final RawJson resJson = (RawJson) result;
        assertThat(resJson.getValue(), containsString("First"));
        assertThat(resJson.getValue(), containsString("Second"));
        assertThat(resJson.getValue(), containsString("Third"));
    }

    @Test
    public void getOptionsLoadsOptionsBySendingQueryToRemoteService() throws Exception {
        final String type = "eventType";
        mockServer.expect(requestTo(expectedUrl("query/" + type + ".sparql"))).andExpect(method(HttpMethod.GET))
                  .andRespond(withSuccess(DATA, MediaType.APPLICATION_JSON));
        final Object result = service.getOptions(type);
        assertNotNull(result);
        assertTrue(result instanceof RawJson);
        final RawJson resJson = (RawJson) result;
        assertEquals(DATA, resJson.getValue());
    }

    private Matcher<String> expectedUrl(String fileName) throws Exception {
        final String sparql = Environment.loadData(fileName);
        return containsString("?query=" + URLEncoder.encode(sparql, Constants.UTF_8_ENCODING));
    }

    @Test
    public void getOptionsThrowsIllegalArgumentForUnknownOption() {
        final String type = "unknownType";
        thrown.expect(IllegalArgumentException.class);
        thrown.expectMessage("Unsupported option type " + type + ".");
        service.getOptions(type);
    }
}